/***********************************************************************

Copyright (c) 2001-2004 Fritz Ganter <ganter@ganter.at>
Website: www.gpsdrive.de

Modifications for ipaq specific version
Copyright (c) 2007 Andrew M. Bishop <amb@gedanken.demon.co.uk>
Website: http://www.gedanken.demon.co.uk/gpsdrive-ipaq/

Disclaimer: Please do not use for navigation.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*********************************************************************/


/*
 * Track support module: display, disk loading/saving.
 */

#include <time.h>
#include <string.h>
#include <sys/time.h>
#include <sys/stat.h>
#include "gpsdrive.h"
#include "track.h"

extern gint maploaded;
extern gint trackflag;
extern glong tracknr;
extern glong trackcoordnr;
extern trackcoordstruct *trackcoord;
extern gint zoom;
extern GdkSegment *track;

extern gint showroute, routeitems;
typedef struct
{
  gchar name[40];
  gdouble lat;
  gdouble longitude;
  gdouble dist;
  gchar typ[40];
  gint action;
}
wpstruct;

extern wpstruct *routelist;
extern GdkColor blue;
extern gint nightmode, isnight, disableisnight;
extern GdkColor red;
extern GdkColor blue;
extern GdkColor trackcolorv;

extern gint real_screen_x;
extern gint real_screen_y;

extern GdkGC *kontext;

extern gchar homedir[500];
extern gchar savetrackfn[256];

extern GdkDrawable *drawable;

/*  if zoom, xoff, yoff or map are changed */
void
rebuildtracklist (void)
{
  gint posxdest, posydest;
  gint i;

  if (!maploaded)
    return;

  if (!trackflag)
    return;

  tracknr = 0;
  for (i = 0; i < trackcoordnr; i++)
    {
      calcxy (&posxdest, &posydest, (trackcoord + i)->longi,
	      (trackcoord + i)->lat, zoom);

/* 	    if even  */
/*       if ((tracknr & 1) == 0)  */
      if (tracknr == 0)
	{
	  if ((trackcoord + i)->longi < 1000.0)
	    {
	      (track + tracknr)->x1 = posxdest;
	      (track + tracknr)->y1 = posydest;
	      tracknr++;
	    }
	}
      else
	{
	  if ((trackcoord + i)->longi < 1000.0)
	    {
	      if ((posxdest > -50) && (posxdest < (SCREEN_X + 50))
		  && (posydest > -50) && (posydest < (SCREEN_Y + 50)))
		{
		  if ((posxdest != (track + tracknr - 1)->x2)
		      || (posydest != (track + tracknr - 1)->y2))
		    {
		      (track + tracknr)->x1 = (track + tracknr - 1)->x2 =
			posxdest;
		      (track + tracknr)->y1 = (track + tracknr - 1)->y2 =
			posydest;
		      tracknr += 1;
		    }
		}
	    }
	  else
	    tracknr = tracknr & ((glong) - 2);
	}

    }

}


/* draw track on image */
void
drawtracks (void)
{
  gint t;
  GdkSegment *routes;

  gint posxdest, posydest;
  gint i, j;

/*    if (!maploaded) */
/*      return; */
  if (!trackflag)
    return;
  if (showroute)
    {
      if (routeitems > 0)
	{
	  i = (routeitems + 5);
	  routes = g_new0 (GdkSegment, i);

	  for (j = 0; j <= routeitems; j++)
	    {
	      calcxy (&posxdest, &posydest, (routelist + j)->longitude,
		      (routelist + j)->lat, zoom);

	      if (j > 0)
		{
		  (routes + j - 1)->x2 = posxdest;
		  (routes + j - 1)->y2 = posydest;
		}
	      (routes + j)->x1 = posxdest;
	      (routes + j)->y1 = posydest;
	    }
	  t = routeitems - 1;
	  gdk_gc_set_line_attributes (kontext, 4, GDK_LINE_ON_OFF_DASH, 0, 0);

	  gdk_gc_set_foreground (kontext, &blue);
	  gdk_draw_segments (drawable, kontext, (GdkSegment *) routes, t);
	  g_free (routes);
	}
    }
  t = 2 * (tracknr >> 1) - 1;
/*     t=tracknr;  */
  if (t < 1)
    return;


  gdk_gc_set_line_attributes (kontext, 4, 0, 0, 0);

  if ((!disableisnight)
      && ((nightmode == 1) || ((nightmode == 2) && isnight)))
    gdk_gc_set_foreground (kontext, &red);
  else
    gdk_gc_set_foreground (kontext, &trackcolorv);

  gdk_draw_segments (drawable, kontext, (GdkSegment *) track, t);

  return;

}


/* stores the track into a file, if argument testname is true, no saving is 
   done, only savetrackfn is set */
void
savetrackfile (gint testname)
{
  struct stat sbuf;
  gchar buff[1024];
  gint e, i;
  gchar mappath[400], lat[30], alt[30], longi[30], timestr[30];
  FILE *st;
  time_t t;
  struct tm *ts;

  if (testname)
    {
      i = 0;
      do
	{
	  g_snprintf (buff, sizeof (buff), "%strack%04d.sav", homedir, i++);
	  e = stat (buff, &sbuf);
	}
      while (e == 0);
      g_strlcpy (savetrackfn, g_basename (buff), sizeof (savetrackfn));
      return;
    }

/* save in new file */
  g_strlcpy (mappath, homedir, sizeof (mappath));
  g_strlcat (mappath, savetrackfn, sizeof(mappath));
  st = fopen (mappath, "w");
  if (st == NULL)
    {
      perror (mappath);
      return;
    }

  for (i = 0; i < trackcoordnr; i++)
    {
      time_t t;
      g_snprintf (lat, sizeof (lat), "%10.6f", (trackcoord + i)->lat);
      g_strdelimit (lat, ",", '.');
      g_snprintf (longi, sizeof (longi), "%10.6f", (trackcoord + i)->longi);
      g_strdelimit (longi, ",", '.');
      g_snprintf (alt, sizeof (alt), "%10.0f", (trackcoord + i)->alt);

      t = (trackcoord + i)->unixtime;
      ts = localtime (&t);
      strncpy (timestr, asctime (ts), 30);
      timestr[strlen (timestr) - 1] = '\0';	/* get rid of \n */

      fprintf (st, "%s %s %s %s\n", lat, longi, alt, timestr);
    }
  fclose (st);

/* append to existing backup file */
  g_strlcpy (mappath, homedir, sizeof (mappath));
  g_strlcat (mappath, "track-ALL.sav", sizeof(mappath));
  st = fopen (mappath, "a");
  if (st == NULL)
    {
      perror (mappath);
      return;
    }

  for (i = 0; i < trackcoordnr; i++)
    {
      g_snprintf (lat, sizeof (lat), "%10.6f", (trackcoord + i)->lat);
      g_strdelimit (lat, ",", '.');
      g_snprintf (longi, sizeof (longi), "%10.6f", (trackcoord + i)->longi);
      g_strdelimit (longi, ",", '.');
      g_snprintf (alt, sizeof (alt), "%10.0f", (trackcoord + i)->alt);

      t = (trackcoord + i)->unixtime;
      ts = localtime (&t);
      strncpy (timestr, asctime (ts), 30);
      timestr[strlen (timestr) - 1] = '\0';	/* get rid of \n */

      fprintf (st, "%s %s %s %s\n", lat, longi, alt, timestr);
    }
  fclose (st);

}
